import { $canFixed } from '@/utils/doms';
import { RESPONSIVE_BREAKPOINTS } from './variables';

const { TABLET, DESKTOP } = RESPONSIVE_BREAKPOINTS;

const { body, documentElement: html } = document;

const freezeClass = '--freeze';
const addBorderClass = '--add-fixed-border';

let lockTimeout;
let lastScroll;
let $header = $('.header');

function calculateScrollWidth() {
  const div = document.createElement('div');

  div.style.position = 'absolute';
  div.style.top = '0px';
  div.style.left = '0px';
  div.style.width = '100%';
  div.style.height = '50px';

  body.appendChild(div);

  const fullWidth = div.offsetWidth;

  div.style.overflowY = 'scroll';

  const limitWidth = div.clientWidth;

  body.removeChild(div);

  const scrollWidth = fullWidth - limitWidth;

  html.classList.add(`--scroll-${scrollWidth}`);

  return scrollWidth;
}

export const removeAccents = (alias) => {
  let str = alias;
  str = str.replace(/à|á|ạ|ả|ã|â|ầ|ấ|ậ|ẩ|ẫ|ă|ằ|ắ|ặ|ẳ|ẵ/g, 'a')
    .replace(/è|é|ẹ|ẻ|ẽ|ê|ề|ế|ệ|ể|ễ/g, 'e')
    .replace(/ì|í|ị|ỉ|ĩ/g, 'i')
    .replace(/ò|ó|ọ|ỏ|õ|ô|ồ|ố|ộ|ổ|ỗ|ơ|ờ|ớ|ợ|ở|ỡ/g, 'o')
    .replace(/ù|ú|ụ|ủ|ũ|ư|ừ|ứ|ự|ử|ữ/g, 'u')
    .replace(/ỳ|ý|ỵ|ỷ|ỹ/g, 'y')
    .replace(/đ/g, 'd')
    .replace(/À|Á|Ạ|Ả|Ã|Â|Ầ|Ấ|Ậ|Ẩ|Ẫ|Ă|Ằ|Ắ|Ặ|Ẳ|Ẵ/g, 'A')
    .replace(/È|É|Ẹ|Ẻ|Ẽ|Ê|Ề|Ế|Ệ|Ể|Ễ/g, 'E')
    .replace(/Ì|Í|Ị|Ỉ|Ĩ/g, 'I')
    .replace(/Ò|Ó|Ọ|Ỏ|Õ|Ô|Ồ|Ố|Ộ|Ổ|Ỗ|Ơ|Ờ|Ớ|Ợ|Ở|Ỡ/g, 'O')
    .replace(/Ù|Ú|Ụ|Ủ|Ũ|Ư|Ừ|Ứ|Ự|Ử|Ữ/g, 'U')
    .replace(/Ỳ|Ý|Ỵ|Ỷ|Ỹ/g, 'Y')
    .replace(/Đ/g, 'D')
    .replace(/\u0300|\u0301|\u0303|\u0309|\u0323/g, '')
    .replace(/\u02C6|\u0306|\u031B/g, '')
    .replace(/[\u0300-\u036f]/g, '');
  return str;
};

export const removeEscape = (value) => {
  const str = value.trim().replace(/(<([^>]+)>)|&|<|>|"|'/ig, '');

  return removeAccents(str);
};

export const regexEmail = /^([A-Za-z0-9][\._\+\-]{0,1})+[A-Za-z0-9]+@[A-Za-z0-9]+(\.{0,1}[A-Za-z0-9]){2,3}\.[a-z]{2,3}$/; //eslint-disable-line

export const scrollWidth = calculateScrollWidth();
export const lang = html.getAttribute('lang') || '';
export const isRTL = html.getAttribute('dir') === 'rtl';
export const isIOS = html.classList.contains('ios');
export const isIE = html.classList.contains('ie')
  || html.classList.contains('edge');

export default {
  scrollWidth,
  lang,
  isRTL,
  isIOS,
  isIE,

  get screenWidth() {
    return window.innerWidth;
  },

  get width() {
    return body.clientWidth;
  },

  get height() {
    return window.innerHeight;
  },

  get bodyHeight() {
    return body.offsetHeight;
  },

  get isFrozen() {
    return body.classList.contains(freezeClass);
  },

  get isDesktop() {
    return this.screenWidth >= DESKTOP;
  },

  get isSmallScreen() {
    return this.screenWidth >= TABLET;
  },

  get isMobile() {
    return !this.isDesktop;
  },

  get scroll() {
    return document.documentElement.scrollTop
      || document.body.scrollTop
      || window.pageYOffset;
  },

  set scroll(top) {
    window.scrollTo({
      top,
      left: 0,
      behavior: 'smooth',
    });
  },

  scrollImmediate(top) {
    window.scrollTo(0, top);
  },

  scrollToElement($element, immediately) {
    if (!$element[0]) {
      return;
    }

    if (!$header[0]) {
      $header = $('.header');
    }

    const scrollTo = $element.offset().top - ($header.outerHeight() || 0);

    if (immediately) {
      this.scrollImmediate(scrollTo);
    } else {
      this.scroll = $element.offset().top - ($header.outerHeight() || 0);
    }
  },

  freeze(callback) {
    clearTimeout(lockTimeout);
    setTimeout(() => {
      window.isFreezing = true;

      const willBeFrozen = !this.isFrozen;

      if (!willBeFrozen) {
        if (typeof callback === 'function') {
          callback();
        }

        return;
      }

      if (this.isIOS) {
        lastScroll = this.scroll;
        body.style.top = `${-lastScroll}px`;
      }

      body.classList.add(freezeClass);

      if (this.bodyHeight > this.height) {
        body.classList.add(addBorderClass);
        $canFixed.each((_, element) => {
          if (window.getComputedStyle(element).position === 'fixed') {
            element.classList.add(addBorderClass);
          }
        });
      }

      if (this.isIOS) {
        this.scrollImmediate(0);
      }

      if (typeof callback === 'function') {
        callback();
      }
    });
  },

  unfreeze(callback) {
    clearTimeout(lockTimeout);
    lockTimeout = setTimeout(() => {
      window.isFreezing = false;

      if (!this.isFrozen) {
        if (typeof callback === 'function') {
          callback();
        }

        return;
      }

      body.classList.remove(freezeClass);
      body.classList.remove(addBorderClass);
      $canFixed.removeClass(addBorderClass);

      if (this.isIOS) {
        body.style.top = '';
        this.scrollImmediate(lastScroll);

        if (typeof callback === 'function') {
          setTimeout(() => {
            callback();
          }, 50);
        }
      } else if (typeof callback === 'function') {
        callback();
      }
    });
  },
};
